-----------------------------------------------------------------------------------------------
-- Create Date:    14:23:54 05/02/2012
-----------------------------------------------------------------------------------------------

library ieee;
library work;

use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.config.all;

entity adc_0820 is
------------------------------------------------------------------------------------------------
--It takes about 1.5us for a analog-to-digital conversion in ADC0820. In this project we choose 
--800ns for the most significant 4 bits and 800ns for the least significant bits. Since the 
--FPGA SPARTAN6 has a inertia frequency of 100Mhz, 800ns needs 80 clocks.
--After practical testing the ADC0820, 800ns is not suficient, that is why N is chosen as 110.
--since the max frequency of ramp (8 bits, 256 levels, triangular) is 100 Hz, 
--each level lasts for 19.5us, that is why we capture and sum 8 converted 
--values for each level which costs 1.1 * 2 * 8 = 17.6us < 19.5us. Then we 
--calculate the average by shiting the sum 3 bits to the right.  
------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------
--!!!ATENTION: WITH THE CONVERTER PMOD_AD2, 8 ACQUISITION IS NOT POSSIBLE DURANTE ONE DEGREE OF RAMP!!!
------------------------------------------------------------------------------------------------
    generic (N: integer := 110);
    Port (
      clk        : in  std_logic;
      reset      : in  std_logic;
      adc_signal : in  std_logic_vector(7 downto 0);   -- 8 pinouts of ADC0820
      acq_tick       : in  std_logic;                  -- acq_tick coming from the DAC done signal
      mode       : in  std_logic;                      -- '0': single acq, '1': average 8 acq.
      output     : out unsigned (7 downto 0);          -- controled output of the output of adc0820
      adc_done   : out std_logic;                      -- pulse signal when a new output is available
      wr         : out std_logic                       -- signal to control ADC0820
    );
end adc_0820;

architecture Behavioral of adc_0820 is

-- signals for FSMD
type state_type is (idle, msb, lsb, average); 
signal state_reg , state_next  : state_type; 
signal sum_reg   , sum_next    : unsigned (10 downto 0);  
signal count_reg , count_next  : unsigned (3 downto 0);   -- it can be 1 or 8
signal output_reg, output_next : unsigned (7 downto 0); 
signal t_reg, t_next           : unsigned (6 downto 0);   -- count for N clock 2^^7=128
signal aux_done_reg, aux_done_next : std_logic;

-- signals to avoid glitches of outputs
signal aux_wr                      : std_logic;  

begin
            
-- FSMD register
process (clk, reset)
begin
    if rising_edge(clk) then
        if reset = '1' then
            state_reg       <= idle; 
            sum_reg         <= (others => '0'); 
            t_reg           <= to_unsigned(1, 7); 
            count_reg       <= to_unsigned(1, 4);
            output_reg      <= (others => '0');
            aux_done_reg    <= '0';
        else
            state_reg       <= state_next; 
            sum_reg         <= sum_next; 
            t_reg           <= t_next; 
            count_reg       <= count_next; 
            output_reg      <= output_next;
            aux_done_reg    <= aux_done_next;
        end if; 
    end if;
end process; 
    
-- FSMD next logic
process(state_reg, sum_reg, count_reg, output_reg, t_reg, adc_signal, acq_tick, mode)
begin
    state_next        <= state_reg; 
    sum_next          <= sum_reg; 
    t_next            <= t_reg; 
    count_next        <= count_reg; 
    output_next       <= output_reg;
    aux_done_next     <= '0';
        
    case state_reg is
        when idle => 
            aux_wr <= '1';
                
            if acq_tick = '1' then
                sum_next   <= (others => '0'); 
                t_next     <= to_unsigned(1, 7); 
                count_next <= to_unsigned(1, 4); 
                state_next <= msb; 
            end if; 
                
        when msb =>
            aux_wr <= '0';
                
            if t_reg = N then 
                t_next     <= to_unsigned(1, 7); 
                state_next <= lsb; 
            else
                t_next <= t_reg + 1; 
            end if; 
                
        when lsb =>
            aux_wr <= '1';
            if t_reg = N then
                t_next <= to_unsigned(1, 7); 
                sum_next <= sum_reg + unsigned(adc_signal); 
            
                if mode = '0' then
                    output_next   <= unsigned(adc_signal); 
                    aux_done_next <= '1'; 
                    state_next    <= idle; 
                else
                    if count_reg = 8 then
                        state_next <= average;
                    else
                        count_next <= count_reg + 1; 
                        state_next <= msb; 
                    end if; 
                end if; 
            else
                t_next <= t_reg + 1; 
            end if; 
         
        when average =>
            aux_wr        <= '1'; 
            output_next   <= sum_reg(10 downto 3); 
            aux_done_next <= '1'; 
            state_next    <= idle; 
  
    end case; 
                
end process; 
        
-- FSMD output
output   <= output_reg; 
adc_done <= aux_done_reg; 
wr       <= aux_wr;            

end Behavioral;
